/*
	DIARIO.C
	This code allows to modify play_rec.dat in order to store the
	game time in Wii's log correctly.

	by Marc
	Thanks to tueidj for giving me some hints on how to do it :)
	Most of the code was taken from here:
	http://forum.wiibrew.org/read.php?27,22130


	Modification by Dr. Clipper:
	now it starts counting the time when you boot the game
*/

#include <stdio.h>
#include <ogcsys.h>

#define SECONDS_TO_2000 946684800LL
#define TICKS_PER_SECOND 60750000LL

#define PLAYRECPATH "/title/00000001/00000002/data/play_rec.dat"

typedef struct{
	u32 checksum;
	union{
		u32 data[0x1f];
		struct{
			char name[84];	//u16 name[42];
			u64 ticks_boot;
			u64 ticks_last;
			char title_id[6];
			u8 unknown[18];
		} ATTRIBUTE_PACKED;
	};
} playrec_struct;



u64 getWiiTime(void) {
	time_t uTime;
	uTime = time(NULL);
	return TICKS_PER_SECOND * (uTime - SECONDS_TO_2000);
}

int Diario_ActualizarDiario(char* bannerTitle,char* gameId){
	s32 ret, playrec_fd;
	u32 sum = 0;
	u32 i;
	u64 stime;
	playrec_struct playrec_buf;

	// CREATE NEW PLAY_REC
	// Update channel name and ID
	for(i=0; i<84; i++)
		playrec_buf.name[i]=bannerTitle[i];
	for(i=0; i<6; i++)
		playrec_buf.title_id[i]=gameId[i];

	// Count from game boot (instead of launcher)
	stime = getWiiTime();
	playrec_buf.ticks_boot=stime;
	playrec_buf.ticks_last=stime;

	// Set unknown bytes to 0
	for(i=0; i<18; i++)
		playrec_buf.unknown[i]=0;

	// Calculate and update checksum
	for(i=0; i<0x1f; i++)
		sum += playrec_buf.data[i];
	playrec_buf.checksum=sum;


	// Save new play_rec.dat
	ret = ISFS_Initialize();
	if(ret){
		//ISFS_Deinitialize();
		printf("* ERROR: could not initialize ISFS (ret=%d)\n",ret);
		return -1;
	}

	playrec_fd = IOS_Open(PLAYRECPATH, IPC_OPEN_WRITE);
	if(playrec_fd<0){
		ISFS_Deinitialize();
		printf("* ERROR: could not open play_rec.dat (ret=%d)\n",playrec_fd);
		return playrec_fd;
	}

	ret = IOS_Write(playrec_fd, &playrec_buf, sizeof(playrec_buf));
	if(ret!=sizeof(playrec_buf)){
		IOS_Close(playrec_fd);
		ISFS_Deinitialize();
		printf("* ERROR: could not write play_rec.dat (ret=%d)\n",ret);
		return -1;
	}
       
	IOS_Close(playrec_fd);
	ISFS_Deinitialize();

	return 0;
}
