#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ogcsys.h>
#include <asndlib.h>
#include <network.h>
#include <sys/errno.h>
#include <unistd.h> //para sleep

#include "disc.h"
#include "usbstorage.h"
#include "wdvd.h"
#include "fat.h"
#include "menu.h"
#include "partition.h"
#include "selectdol.h"
#include "sys.h"
#include "utils.h"
#include "video.h"
#include "wbfs.h"
#include "wpad.h"
#include "sonido.h"
#include "diario.h"
#include "http.h"
#include "langs.h"

// Constants
#define ARROWS_X		16
#define ARROWS_Y		228
#define ARROWS_WIDTH	20
#define MIGHTY_PATH		"fat:/config/mighty/"
#define IMAGES_PREFIX	"covers"
#define MIGHTY_CONFIG_FILE	"mighty_loader.cfg"
#define MAXJUEGOS		800
#define EMPTY			-1

#define BLACK	0x000000FF
#define WHITE	0xFFFFFFFF
#define BLUE	0x27347bff

// Gamelist buffer & variables
static struct discHdr *gameList = NULL;
static s32 gameCnt = 0;

// Variables for MIGHTY
static u16 paginasLeidas, pagina, maxPages;
static int selectedGame=0, movingGame=-1;
static const char** lang;
static int loadingAnim=0;
static bool wideScreen=false;
static bool haHabidoCambios=false;
static char tempString[128];


typedef struct {
    u8 modoVideo;
    bool anti002;
    u8 ocarina;
    u8 altdol;
    u8 ios;
    MRCtex* banner;
} configuracionJuego;
static configuracionJuego* configuracionJuegos=NULL;
static s16* orden;


#include "mighty_loader_arrows_png.h"
#include "mighty_loader_background_png.h"
#include "mighty_loader_container_png.h"
#include "mighty_loader_container_wide_png.h"
#include "mighty_loader_loading_png.h"
#include "mighty_loader_pages_png.h"

#define MAX_TEXTURES	5
#define TEX_ARROWS	0
#define TEX_BACKGROUND	1
#define TEX_CONTAINER	2
#define TEX_LOADING	3
#define TEX_PAGES		4
static MRCtex* textures[MAX_TEXTURES];


const u8 COLS[]={4, 5};
#define ROWS 2
#define FIRSTCOL 92
#define FIRSTROW 128
const u8 SEPARACIONX[]={152, 112};
#define SEPARACIONY 198
const u8 ANCHOIMAGEN[]={128, 96};
#define ALTOIMAGEN 176



void __Draw_Loading(void){
 	MRC_Draw_Tile(592,432, textures[TEX_LOADING], 24, loadingAnim);
	__Render_Box(592,432);

	loadingAnim++;
	if(loadingAnim==16)
		loadingAnim=0;
}

void __Draw_Page(int selected){
	int i, j, x, y, containerWidth, game;

	containerWidth=textures[TEX_CONTAINER]->width/2;

	// Background
	MRC_Draw_Texture(0, 0, textures[TEX_BACKGROUND]);

	if(gameCnt==0 || paginasLeidas==0){
		return;
	}

	// Games
	game=COLS[wideScreen]*ROWS*pagina;
	y=FIRSTROW;
	for(i=0; i<ROWS; i++){
		x=FIRSTCOL;
		for(j=0; j<COLS[wideScreen]; j++){
			if(movingGame!=game){
				/*if(orden[game]==EMPTY){
					MRC_Draw_Texture(x, y, textures[TEX_EMPTY]);
					MRC_Draw_Tile(x, y, textures[TEX_CONTAINER], containerWidth, 0);
				}else if(selected==i*COLS[wideScreen]+j){
					MRC_Draw_Texture(x, y, configuracionJuegos[orden[game]].banner);
					MRC_Draw_Tile(x, y, textures[TEX_CONTAINER], containerWidth, 1);
				}else{
					MRC_Draw_Texture(x, y, configuracionJuegos[orden[game]].banner);
					MRC_Draw_Tile(x, y, textures[TEX_CONTAINER], containerWidth, 0);
				}*/
				if(orden[game]!=EMPTY){
					MRC_Draw_Texture(x, y, configuracionJuegos[orden[game]].banner);
					MRC_Draw_Tile(x, y, textures[TEX_CONTAINER], containerWidth, (selected==i*COLS[wideScreen]+j));
				}
			}
			game++;
			x+=SEPARACIONX[wideScreen];
		}
		y+=SEPARACIONY;
	}

	// Page number
	x=(640-(textures[TEX_PAGES]->width/2*(maxPages-1)))/2;
	for(i=0; i<maxPages; i++){
		MRC_Draw_Tile(x, 440, textures[TEX_PAGES], textures[TEX_PAGES]->width/2, i==pagina);
		x+=textures[TEX_PAGES]->width/2;
	}

	//MRC_Draw_String(300, 404, 0x808080FF, tempString);
	//MRC_Draw_String(310, 404, 0xd0d0d0FF, tempString);
	/*sprintf(tempString, "hotspot=%d", selected);
	MRC_Draw_String(40, 444, 0xFFFF00FF, tempString);*/

	if(movingGame>-1){
		if(orden[movingGame]!=EMPTY){
			MRC_Draw_Texture(Wpad_GetWiimoteX(), Wpad_GetWiimoteY(), configuracionJuegos[orden[movingGame]].banner);
		}
	}

	// Arrows
	MRC_Draw_Tile(ARROWS_X, ARROWS_Y, textures[TEX_ARROWS], ARROWS_WIDTH, 0+(pagina>0)+(pagina>0 && selected==HOTSPOT_LEFT));
	MRC_Draw_Tile(640-ARROWS_X, ARROWS_Y, textures[TEX_ARROWS], ARROWS_WIDTH, 3+(pagina+1<maxPages)+(pagina+1<maxPages && selected==HOTSPOT_RIGHT));
}

void __Draw_Button(int hot, const char* text, bool selected){
	hotSpot button=Wpad_GetHotSpotInfo(hot);
	int textX=button.x+(button.width-MRC_Get_String_Width(text))/2;
	u32 color;
	if(selected){
		__Draw_Box(button.x, button.y, button.width, button.height/2, 0x3c7291ff);
		__Draw_Box(button.x, button.y+button.height/2, button.width, button.height/2, 0x2d6483ff);
		color=WHITE;
	}else{
		__Draw_Box(button.x, button.y, button.width, button.height/2, 0xe3e3e3ff);
		__Draw_Box(button.x, button.y+button.height/2, button.width, button.height/2, 0xd8d8d8ff);
		__Draw_Box(button.x, button.y, button.width, 1, 0xb6b4c5ff);
		__Draw_Box(button.x, button.y+button.height-1, button.width, 1, 0xb6b4c5ff);
		__Draw_Box(button.x, button.y, 1, button.height, 0xb6b4c5ff);
		__Draw_Box(button.x+button.width-1, button.y, 1, button.height, 0xb6b4c5ff);
		color=0x404040ff;
	}
	MRC_Draw_String(textX, button.y+button.height/2-8, color, text);
}

void __Draw_Window(int width, int height){
	int x=(640-width)/2;
	int y=(480-height)/2;
	__Draw_Page(-1);
	__Draw_Box(0, 0, 640, 480, 0x000000c0);

	__Draw_Box(x, y, width, height, WHITE-0x10);
}

#define QUESTION_BUTTON_X			100
#define QUESTION_BUTTON_Y			240
#define QUESTION_BUTTON_SEPARATION	32
#define QUESTION_BUTTON_WIDTH		200
#define QUESTION_BUTTON_HEIGHT		48
bool __Question_Window(const char* text, const char* a1, const char* a2){
	int i, hotSpot, hotSpotPrev;
	bool ret=false, repaint=true;

	// Create/restore hotspots
	Wpad_CleanHotSpots();
	for(i=0; i<2; i++)
		Wpad_AddHotSpot(i,
			QUESTION_BUTTON_X+i*(QUESTION_BUTTON_WIDTH+QUESTION_BUTTON_SEPARATION),
			QUESTION_BUTTON_Y,
			QUESTION_BUTTON_WIDTH,
			QUESTION_BUTTON_HEIGHT,
			i, i,
			!i, !i
		);


	__Draw_Window(512, 128);
	MRC_Draw_String(100, 200, BLACK, text);

	// Loop
	hotSpot=hotSpotPrev=-1;

	ret=MENU_SELECT_GAME;
	for(;;){
		hotSpot=Wpad_Scan();

		// If hot spot changed
		if((hotSpot!=hotSpotPrev && hotSpot<2) || repaint){
			hotSpotPrev=hotSpot;

			__Draw_Button(0, a1, hotSpot==0);
			__Draw_Button(1, a2, hotSpot==1);

			repaint=false;
		}
		__Draw_Cursor(Wpad_GetWiimoteX(), Wpad_GetWiimoteY(), 0);

		if(((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_A) || (PAD_ButtonsDown(0) & PAD_BUTTON_A)) && hotSpot!=-1){
			if(hotSpot==1)
				ret=true;
			break;
		}
	}

	return ret;
}

void calcularPaginas(void){
	int i;
	maxPages=0;
	for(i=MAXJUEGOS-1; i>-1; i--){
		if(orden[i]!=EMPTY){
			break;
		}
	}

	while(maxPages*COLS[wideScreen]*ROWS<=i){
		maxPages++;
	}

	if(movingGame!=-1)
		maxPages++;

	if(maxPages>50)
		maxPages=50;
}


s32 __Menu_GetEntries(void){
	struct discHdr *buffer = NULL;
	
	u32 cnt, len;
	int i, j, k, ret;

	// Get list length
	ret = WBFS_GetCount(&cnt);
	if (ret < 0)
		return ret;

	// Buffer length
	len = sizeof(struct discHdr) * cnt;

	// Allocate memory
	buffer = (struct discHdr *)memalign(32, len);
	if (!buffer)
		return -1;

	// Clear buffer
	memset(buffer, 0, len);

	// Get header list
	ret = WBFS_GetHeaders(buffer, cnt, sizeof(struct discHdr));
	if (ret < 0)
		goto err;

	// Free memory
	if (gameList)
		free(gameList);

	// Set values
	gameList = buffer;
	gameCnt  = cnt;

	// Reservar espacio para configuraciones
	if(configuracionJuegos)
		free(configuracionJuegos);
	configuracionJuegos=malloc(sizeof(configuracionJuego)*gameCnt);

	if(orden)
		free(orden);
	orden=malloc(sizeof(u16)*MAXJUEGOS);

	s16* posiciones=malloc(sizeof(u16)*gameCnt);

	for(i=0; i<gameCnt; i++){
		posiciones[i]=-1;
		configuracionJuegos[i].modoVideo=0;
		configuracionJuegos[i].anti002=true;
		configuracionJuegos[i].ocarina=0;
		configuracionJuegos[i].altdol=0;
		configuracionJuegos[i].ios=0;
	}

	
	// Ordenar los juegos segun archivo de configuracion
	char *archivoLeido=NULL;

	ret = Fat_ReadFile(MIGHTY_PATH MIGHTY_CONFIG_FILE, (void *)&archivoLeido);


	if(ret>0){
		// Parse config file
		i=16;
		while(i<ret){
			j=archivoLeido[i]+archivoLeido[i+1]*256;

			for(k=0; k<6; k++)
				tempString[k]=archivoLeido[i+2+k];
			tempString[6]='\0';

			for(k=0; k<gameCnt; k++){
				struct discHdr *header = &gameList[k];
				if(strcmp(tempString,(void *)header->id)==0){
					//printf("%s found at %d\n", tempString, k);
					configuracionJuegos[k].modoVideo=archivoLeido[i+8];
					configuracionJuegos[k].anti002=archivoLeido[i+9];
					configuracionJuegos[k].ocarina=archivoLeido[i+10];
					configuracionJuegos[k].altdol=archivoLeido[i+11];
					configuracionJuegos[k].ios=archivoLeido[i+12];

					posiciones[k]=j;
					break;
				}
			}
			i+=16;
		}
		free(archivoLeido);
	}

	//Initialize empty
	for(i=0; i<MAXJUEGOS; i++)
		orden[i]=EMPTY;

	//Place games
	for(i=0; i<gameCnt; i++)
		if(posiciones[i]!=-1)
			orden[posiciones[i]]=i;

	//Check if some game was not placed
	for(i=0; i<gameCnt; i++){
		if(posiciones[i]==-1){
			for(j=0; j<MAXJUEGOS; j++){
				if(orden[j]==EMPTY){
					posiciones[i]=j;
					break;
				}
			}
		}
		orden[posiciones[i]]=i;
	}
	free(posiciones);

	selectedGame=0;
	pagina=0;
	paginasLeidas=0;
	maxPages=0;
	calcularPaginas();

	return 0;

err:
	// Free memory
	if (buffer)
		free(buffer);

	return ret;
}


void __Finish_ALL_GFX(void){
	int i;

	for(i=0; i<MAXJUEGOS; i++){
		//printf("%d: %d\n", i, orden[i]);

		if(i==paginasLeidas*COLS[wideScreen]*ROWS)
			break;
		if(orden[i]!=EMPTY){
			__Free_Texture(configuracionJuegos[orden[i]].banner);
		}
	}

	for(i=0; i<MAX_TEXTURES; i++)
		__Free_Texture(textures[i]);

	MRC_Finish();

	haHabidoCambios=false;
}

void __Load_Images_From_Page(void){
	void *imgBuffer=NULL;
	struct discHdr *header;
	int i, max, ret, game;

	#ifdef DEBUG_MODE
	printf("Loading images...\n");
	#endif

	max=COLS[wideScreen]*ROWS;


	for(i=0; i<max; i++){
		game=orden[max*paginasLeidas+i];
		if(game!=EMPTY){
			__Draw_Loading();
			header=&gameList[game];

			// Load image from FAT
			sprintf(tempString, MIGHTY_PATH IMAGES_PREFIX "/%s.png", header->id);

			ret=Fat_ReadFile(tempString, &imgBuffer);
			// Decode image
			if(ret>0){
				configuracionJuegos[game].banner=MRC_Load_Texture(imgBuffer);
				free(imgBuffer);
			}else{
				configuracionJuegos[game].banner=__Create_No_Banner((char*)header->id, (char*)header->title, ANCHOIMAGEN[wideScreen], ALTOIMAGEN);
			}

			__Resize_Texture(configuracionJuegos[game].banner, ANCHOIMAGEN[wideScreen], ALTOIMAGEN);
			__Center_Texture(configuracionJuegos[game].banner, 1);
		}
		//MRC_Draw_Texture(64, 440, configuracionJuegos[game].banner);
	}
	paginasLeidas++;
}




bool Menu_Format(void){
	partitionEntry partitions[MAX_PARTITIONS];

	u32 cnt, sector_size;
	s32 ret, selected = 0;


	__Finish_ALL_GFX();
	Con_Init(CONSOLE_X, CONSOLE_Y, CONSOLE_WIDTH, CONSOLE_HEIGHT);

	// Clear console
	Con_Clear();
	// Get partition entries
	ret = Partition_GetEntries(WBFS_DEVICE_USB, partitions, &sector_size);
	if(ret<0){
		printf("* ERROR getting partitions (ret = %d)\n", ret);

		// Restart
		Wpad_WaitButtons();
		return false;
	}

loop:
	// Clear console
	Con_Clear();

	printf(lang[13]);
	printf(lang[14]);
	printf(lang[15]);

	// Print partition list
	for (cnt = 0; cnt < MAX_PARTITIONS; cnt++) {
		partitionEntry *entry = &partitions[cnt];

		// Calculate size in gigabytes
		f32 size = entry->size * (sector_size / GB_SIZE);

		// Selected entry
		(selected == cnt) ? printf(">> ") : printf("   ");
		fflush(stdout);

		// Valid partition
		if (size)
			printf("Partition #%d: (size = %.2fGB)\n",       cnt + 1, size);
		else 
			printf("Partition #%d: (cannot be formatted)\n", cnt + 1);
	}

	partitionEntry *entry = &partitions[selected];
	u32           buttons = Wpad_WaitButtons();

	// UP/DOWN buttons
	if (buttons==WPAD_BUTTON_UP) {
		if ((--selected) <= -1)
			selected = MAX_PARTITIONS - 1;
	}

	if (buttons==WPAD_BUTTON_DOWN) {
		if ((++selected) >= MAX_PARTITIONS)
			selected = 0;
	}

	// B button
	if (buttons==WPAD_BUTTON_B)
		return false;

	// Valid partition
	if (entry->size) {
		// A button
		if (buttons==WPAD_BUTTON_A)
			goto format;
	}

	goto loop;

format:
	// Clear console
	Con_Clear();
	printf("  Format this partition to WBFS?\n\n");

	printf("  Partition #%d\n",                  selected + 1);
	printf("  (size = %.2fGB - type: %02X)\n\n", entry->size * (sector_size / GB_SIZE), entry->type);

	printf("  [A] Format this partition\n");
	printf("  [B] Cancel\n\n\n");

	// Wait for user answer
	for (;;) {
		u32 buttons = Wpad_WaitButtons();

		// A button
		if (buttons==WPAD_BUTTON_A)
			break;

		// B button
		if (buttons==WPAD_BUTTON_B)
			goto loop;
	}

	printf(" * Formatting, please wait...\n");
	fflush(stdout);

	// Format partition
	ret = WBFS_Format(entry->sector, entry->size);
	if (ret < 0) {
		printf("    ERROR! (ret = %d)\n", ret);
		goto out;
	} else
		printf(" OK!\n");

out:
	printf(lang[7]);

	// Wait for any button
	Wpad_WaitButtons();
	return true;
}


#define TIMEOUT 30
int Open_Device(void){
	s32 ret;

	// Draw loading HDD message
	sprintf(tempString, lang[8], TIMEOUT);

	MRC_Draw_Texture(0, 0, textures[TEX_BACKGROUND]);
	__Render_Screen();
	__Draw_Loading();

	// Initialize WBFS
	ret = WBFS_Init(WBFS_DEVICE_USB, TIMEOUT);
	
	// Initialize pads
	Wpad_Init();

	while(ret<0){
		sprintf(tempString, "%s (ret=%d)", lang[9], ret);
		if(__Question_Window(tempString, lang[10], lang[12])){
			goto out;
		}

		ret = WBFS_Init(WBFS_DEVICE_USB, TIMEOUT);
	}

	// Try to open device
	if(WBFS_Open()<0){
		Menu_Format();
		ret=-1;
	}

out:
	return ret;
}

/*int __ReloadIOS(int ios){
        int ret = -1;

        // Close storage
        if (usb_m) {
                Fat_UnmountUSB();
                USBStorage_Deinit();
                Fat_UnmountSDHC();
        }

        // Close subsystems
        if (subsys) {
                Subsystem_Close();
                WDVD_Close();
                USBStorage_Deinit();
                SDHC_Close();
        }

        //Load Custom IOS
        usleep(300000);
        ret = IOS_ReloadIOS(ios);
        usleep(300000);
        if (ret < 0) {
                printf("\n[+] ERROR:\n");
                printf("    Custom IOS %d could not be loaded! (ret = %d)\n", CFG.ios, ret);
                goto err;
        }
        printf(".");

        printf(".");

        // re-initialize subsystems

                // wpad
                Wpad_Init();
                // storage
                if (wbfsDev == WBFS_DEVICE_USB) {
                        USBStorage_Init();
                }
                if (wbfsDev == WBFS_DEVICE_SDHC) {
                        SDHC_Init();
                }
                // init dip
                ret = Disc_Init();
                if (ret < 0) {
                        if (verbose) {
                                printf("\n[+] ERROR:\n");
                                printf("    Could not initialize DIP module! (ret = %d)\n", ret);
                        }
                        goto err;
                }

        return 0;

err:
        if (subsys) {
                Wpad_Init();
                Restart_Wait();
        }
        return ret;
}*/







f32 __Menu_GetGameSize(struct discHdr *header){
	f32 size = 0.0;

	// Get game size
	WBFS_GameSize(header->id, &size);

	return size;
}

int __Install(void){
	static struct discHdr header ATTRIBUTE_ALIGN(32);
	bool answer;
	int ret;


	// Ask user
	answer=__Question_Window(lang[16], lang[12], lang[11]);

	if(!answer){
		return MENU_HOME;
	}

	__Finish_ALL_GFX();
	Con_Init(CONSOLE_X, CONSOLE_Y, CONSOLE_WIDTH, CONSOLE_HEIGHT);


	// Disable WBFS mode
	Disc_SetWBFS(0, NULL);

	printf(lang[18]);
	fflush(stdout);

	// Wait for disc
	ret = Disc_Wait();
	if (ret < 0) {
		printf("ERROR! (ret=%d)\n", ret);
		goto out;
	} else
		printf("OK!\n\n");


	printf(lang[20]);
	fflush(stdout);

	// Open disc
	ret = Disc_Open();
	if (ret < 0) {
		printf("ERROR! (ret=%d)\n", ret);
		goto out;
	} else
		printf("OK!\n\n");

	// Check disc
	ret = Disc_IsWii();
	if (ret < 0) {
		printf("ERROR: Not a Wii disc!!\n");
		goto out;
	}

	// Read header
	Disc_ReadHeader(&header);

	// Check if game is already installed
	ret = WBFS_CheckGame(header.id);
	if (ret) {
		printf("ERROR: This game is already installed!\n");
		goto out;
	}

	printf(lang[21]);
	printf("\n\n");

	printf("   %s\n",           header.title);
	printf("   (%c%c%c%c)\n\n", header.id[0], header.id[1], header.id[2], header.id[3]);

	// Install game
	ret = WBFS_AddGame();
	if (ret < 0) {
		printf("ERROR installing (ret=%d)\n", ret);
		goto out;
	}

	printf("\n\n");
	printf(lang[7]);

	// Wait for any button
	Wpad_WaitButtons();

out:

	return MENU_EXIT;
}

int __Uninstall_Game(void){
	struct discHdr *header = NULL;
	u16 game=orden[selectedGame];
	bool answer;
	int ret;

	// No game list
	if (!gameCnt)
		return MENU_SELECT_GAME;

	// Selected game
	header = &gameList[game];

	// Ask user
	sprintf(tempString, "%s %s", lang[35], header->title);
	answer=__Question_Window(tempString, lang[12], lang[19]);
	if(!answer){
		return MENU_SELECT_GAME;
	}
	answer=__Question_Window(lang[22], lang[11], lang[12]);
	if(answer){
		return MENU_SELECT_GAME;
	}

	__Finish_ALL_GFX();
	Con_Init(CONSOLE_X, CONSOLE_Y, CONSOLE_WIDTH, CONSOLE_HEIGHT);

	printf(lang[23]);
	// Remove game
	ret = WBFS_RemoveGame(header->id);
	if (ret < 0) {
		printf("\n    ERROR! (ret = %d)\n", ret);
		goto out;
	} else
		printf(" OK!\n");

out:
	printf(lang[7]);

	// Wait for any button
	Wpad_WaitButtons();

	return MENU_EXIT;
}


void __Save_Changes(void){
	int i, j, configsize;
	unsigned char *outBuffer;

	configsize=1+15+gameCnt*(2+6+5+3); //1=cfgversion + 15=reserved   +   (2=position   6=id   5=config   3=reserved)=16
	#ifdef DEBUG_MODE
	printf("Saving changes... (%d bytes)\n", configsize);
	#endif
	outBuffer=malloc(configsize);

	for(i=0; i<16; i++)
		outBuffer[i]=0;
	outBuffer[0]=CONFIG_VERSION;

	j=16;
	for(i=0; i<MAXJUEGOS; i++){
		__Draw_Loading();
		if(orden[i]!=EMPTY){
			struct discHdr *header = &gameList[orden[i]];
			outBuffer[j]=i%256;
			outBuffer[j+1]=i/256;
			outBuffer[j+2]=header->id[0];
			outBuffer[j+3]=header->id[1];
			outBuffer[j+4]=header->id[2];
			outBuffer[j+5]=header->id[3];
			outBuffer[j+6]=header->id[4];
			outBuffer[j+7]=header->id[5];

			outBuffer[j+8]=configuracionJuegos[orden[i]].modoVideo;
			outBuffer[j+9]=configuracionJuegos[orden[i]].anti002;
			outBuffer[j+10]=configuracionJuegos[orden[i]].ocarina;
			outBuffer[j+11]=configuracionJuegos[orden[i]].altdol;
			outBuffer[j+12]=configuracionJuegos[orden[i]].ios;
			outBuffer[j+13]=0;
			outBuffer[j+14]=0;
			outBuffer[j+15]=0;
			j+=16;
		}
	}

	// Save changes to FAT
	Fat_SaveFile(MIGHTY_PATH MIGHTY_CONFIG_FILE, (void *)&outBuffer, configsize);

	haHabidoCambios=false;
}

void __Load_Skin(bool loadFromSD){
	const char* fileNames[MAX_TEXTURES]={
		"arrows", "background", (wideScreen? "container_wide" : "container"), 
		"loading", "pages"};
	const u8* defaultTextures[MAX_TEXTURES]={
		mighty_loader_arrows_png, mighty_loader_background_png, (wideScreen? mighty_loader_container_wide_png : mighty_loader_container_png), 
		mighty_loader_loading_png, mighty_loader_pages_png};

	int i, ret;
	char *imgData = NULL;

	if(loadFromSD){
		for(i=0; i<MAX_TEXTURES; i++){
			sprintf(tempString, MIGHTY_PATH "mighty_loader_%s.png", fileNames[i]);
			ret = Fat_ReadFile(tempString, (void*)&imgData);
			if(ret>0){
				textures[i]=MRC_Load_Texture(imgData);
				free(imgData);
			}else{ //cargar textura por defecto
				textures[i]=MRC_Load_Texture((void *)defaultTextures[i]);
			}
		}

		__Center_Texture(textures[TEX_ARROWS], 6);
		__Center_Texture(textures[TEX_CONTAINER], 2);
		__Center_Texture(textures[TEX_PAGES], 2);

	}else{
		for(i=0; i<MAX_TEXTURES; i++){
			textures[i]=MRC_Load_Texture((void *)defaultTextures[i]);
		}
	}
}


int __Select_Game(void){
	int i, j, hotSpot, hotSpotPrev, button, ret;
	ret=MENU_EXIT;

	if(gameCnt==0)
		return MENU_HOME;

	// Create/restore hotspots
	for(i=0;i<ROWS;i++){
		for(j=0;j<COLS[wideScreen];j++){
			int pos=i*COLS[wideScreen]+j;
			Wpad_AddHotSpot(pos,
				FIRSTCOL-ANCHOIMAGEN[wideScreen]/2+j*SEPARACIONX[wideScreen],
				FIRSTROW-ALTOIMAGEN/2+i*SEPARACIONY,
				ANCHOIMAGEN[wideScreen],
				ALTOIMAGEN,
				(i==0? COLS[wideScreen]*(ROWS-1)+j : pos-COLS[wideScreen]),
				(i==ROWS-1? j : pos+COLS[wideScreen]),
				(j==0? pos+COLS[wideScreen]-1 : pos-1),
				(j==COLS[wideScreen]-1? pos-COLS[wideScreen]+1 : pos+1)
			);
		}
	}
	Wpad_AddHotSpot(HOTSPOT_LEFT, 0,160,32,88, HOTSPOT_LEFT, HOTSPOT_LEFT, HOTSPOT_RIGHT, 0);
	Wpad_AddHotSpot(HOTSPOT_RIGHT, 640-32,160,32,88, HOTSPOT_RIGHT, HOTSPOT_RIGHT,(COLS[wideScreen]*2)-1, HOTSPOT_LEFT);
	hotSpot=hotSpotPrev=-1;	
	Wpad_SetHotSpot(selectedGame%(ROWS*COLS[wideScreen]));

	// Load images from actual page
	if(paginasLeidas<pagina+1)
		__Load_Images_From_Page();
	__Draw_Page(-1);

	// Select game loop
	for(;;){
		hotSpot=Wpad_Scan();

		// If hot spot changed
		if(movingGame>-1){
			__Draw_Page(hotSpot);
		}else if(hotSpot!=hotSpotPrev){
			hotSpotPrev=hotSpot;
			__Draw_Page(hotSpot);
		}

		if(hotSpot>=0 && hotSpot<=COLS[wideScreen]*ROWS){
			selectedGame=pagina*COLS[wideScreen]*ROWS+hotSpot;
		}else{
			selectedGame=-1;
		}

		__Draw_Cursor(Wpad_GetWiimoteX(), Wpad_GetWiimoteY(), (movingGame>-1));

		button=Wpad_GetButton();
		if(movingGame==-1){
			if(button && hotSpot>-1 && hotSpot<COLS[wideScreen]*ROWS && orden[selectedGame]!=EMPTY){
				if(button & WPAD_BUTTON_B){
					movingGame=selectedGame;
					calcularPaginas();
				}else if(button & WPAD_BUTTON_A){
					ret=MENU_SHOW_GAME;
					break;
				}else if(button & WPAD_BUTTON_1){
					ret=MENU_CONFIG_GAME;
					break;
				}
			}
		}else{
			//moving game
			if(!((WPAD_ButtonsHeld(WPAD_CHAN_0) & WPAD_BUTTON_B) || (PAD_ButtonsHeld(0) & PAD_BUTTON_B))){
				if(selectedGame!=-1 && selectedGame!=movingGame){
					u16 copia0=orden[movingGame];
					u16 copia1=orden[selectedGame];

					orden[movingGame]=copia1;
					orden[selectedGame]=copia0;
					haHabidoCambios=true;
				}
				movingGame=-1;
				calcularPaginas();
				__Draw_Page(hotSpot);
			}
		}

		if(button){
			if(pagina>0 && ((button & (WPAD_BUTTON_MINUS | PAD_TRIGGER_L)) || (hotSpot==HOTSPOT_LEFT && (button & (WPAD_BUTTON_A | PAD_BUTTON_A))))){
				pagina--;
				ret=MENU_SELECT_GAME;
				break;
			}else if(pagina+1<maxPages && ((button & (WPAD_BUTTON_PLUS | PAD_TRIGGER_R)) || (hotSpot==HOTSPOT_RIGHT && (button & (WPAD_BUTTON_A | PAD_BUTTON_A))))){
				pagina++;
				ret=MENU_SELECT_GAME;
				break;
			}if(button & WPAD_BUTTON_HOME){
				ret=MENU_HOME;
				break;
			}

/*
			if(button & WPAD_BUTTON_2){
				MRC_Capture();
			}
*/

		}

	}

	return ret;
}

#define PROJECTION_HEIGHT 128
int __Show_Game(bool autoMode){
	void* imageBuffer;
	MRCtex *gameImage, *projection;
	int i, j, ret;
	char *c, *r, a;
	struct discHdr *header = &gameList[orden[selectedGame]];
	SoundInfo snd;

	
	// BLACK SCREEN
	// BLACK SCREEN
	a=250;
	for(i=0; i<480; i++){		
		if(i<100 && i%2==0){
			a--;
		}else if(i>280 && i%4==0){
				a++;
		}
		__Draw_Box(0, i, 640, 1, a);
	}

	/*
	// ANOTHER SCREEN FADE TYPE
	a=200;
	for(i=0; i<480; i++){
		if(a<255 && ((i<100 && i%4==0) || (i>200 && i%8==0)))
			a++;
		__Draw_Box(0, i, 640, 1, 0x20202000+a);
	}*/


	// Load image from FAT
	sprintf(tempString, MIGHTY_PATH IMAGES_PREFIX "/%s.png", header->id);

	ret=Fat_ReadFile(tempString, &imageBuffer);
	// Decode image
	if(ret>0){
		gameImage=MRC_Load_Texture(imageBuffer);
		free(imageBuffer);

		__Resize_Texture(gameImage, (wideScreen? 168 : 224), 308);
		__Center_Texture(gameImage, 1);

		projection=malloc(sizeof(MRCtex));
		projection->buffer=malloc(gameImage->width*PROJECTION_HEIGHT*4);
		projection->width=gameImage->width;
		projection->height=PROJECTION_HEIGHT;
		__Center_Texture(projection, 1);
		projection->alpha=true;
	
		a=128;
		r=(projection->buffer);

		for(i=0; i<PROJECTION_HEIGHT; i++){
			c=(gameImage->buffer)+(((gameImage->height-1)-i)*gameImage->width)*4;
			for(j=0; j<gameImage->width; j++){
				r[0]=c[0];
				r[1]=c[1];
				r[2]=c[2];
				r[3]=a;
				c+=4;
				r+=4;
			}
			if(a>2)
				a-=2;
		}
	
		MRC_Draw_Texture(320, 228, gameImage);
		MRC_Draw_Texture(320, 446, projection);
		__Free_Texture(gameImage);
		__Free_Texture(projection);
	}else{
		MRC_Draw_String((640-MRC_Get_String_Width(header->title))/2+2, 220, 0x00000000, header->title);
		MRC_Draw_String((640-MRC_Get_String_Width(header->title))/2, 220, 0xFFFFFF00, header->title);
	}
	__Render_Screen();



	//Play banner SOUND
	if(!autoMode){
		ASND_Init();
		ASND_Pause(0);
		
		memset(&snd, 0, sizeof(snd));
		WBFS_BannerSound(header->id, &snd);
		if (snd.dsp_data) {
			int fmt = (snd.channels == 2) ? VOICE_STEREO_16BIT : VOICE_MONO_16BIT;
	
			if(snd.loop){
				ASND_SetInfiniteVoice(0, fmt, snd.rate, 0,			
					snd.dsp_data, snd.size,
					160,160); //volumen,volumen
			}else{
				SND_SetVoice(0, fmt, snd.rate, 0,
					snd.dsp_data, snd.size,
					96,96, //volumen,volumen,
					NULL); //DataTransferCallback
			}
		}
	
		ret=MENU_SELECT_GAME;
		for(;;){
			WPAD_ScanPads();
			PAD_ScanPads();
			if(WPAD_ButtonsDown(WPAD_CHAN_0) || PAD_ButtonsDown(0)){
				if((WPAD_ButtonsDown(WPAD_CHAN_0) & WPAD_BUTTON_A) || PAD_ButtonsDown(0) & PAD_BUTTON_A){
					ret=MENU_START_GAME;
				}
				break;
			}
		}
	
		// Stop banner sound
		if(snd.dsp_data){
			SND_StopVoice(0);
			ASND_End();

			free(snd.dsp_data);
			snd.dsp_data=NULL;
		}
	}else{
		ret=MENU_START_GAME;
	}

	return ret;
}

int __Start_Game(void){
	int ret;
	struct discHdr *header;
	char *gameTitle;

	// No game list
	if (!gameCnt)
		return -1;

	// Selected game
	header=&gameList[orden[selectedGame]];

	gameTitle=WBFS_BannerTitle(header->id);
	if(Diario_ActualizarDiario(gameTitle, (char*)header->id)<0){
		printf("ERROR DIARIO");
		Wpad_WaitButtons();
	}
	free(gameTitle);

	//Reload IOS if needed
	if(configuracionJuegos[orden[selectedGame]].ios>0){
		WDVD_Close();
		USBStorage_Deinit();
		Wpad_Disconnect();
		usleep(3000);
		IOS_ReloadIOS(configuracionJuegos[orden[selectedGame]].ios);
		//Wpad_Init();
		Disc_Init();
		Open_Device();
		//Fat_MountSD();
		//__Menu_GetEntries();
		//Fat_Unmount();
	}

	//printf("Ejecutando juego, por favor espera...\n");

	/* Set WBFS mode */
	Disc_SetWBFS(WBFS_DEVICE_USB, header->id);

	/* Open disc */
	ret = Disc_Open();
	if(ret < 0){
		printf("    ERROR: Could not open game! (ret = %d)\n", ret);
		goto out;
	}

	/* Boot Wii disc */
	u8 modoVideo=configuracionJuegos[orden[selectedGame]].modoVideo;

	bool anti002=configuracionJuegos[orden[selectedGame]].anti002;

	u16 altdol=configuracionJuegos[orden[selectedGame]].altdol;
	if(altdol>0)
		altdol=select_dol_from_internal_db((void *)header->id, altdol)->number;

	u8 ocarina=configuracionJuegos[orden[selectedGame]].ocarina;

	ret=Disc_WiiBoot(modoVideo, anti002, ocarina, altdol);

	printf("    Returned! (ret = %d)\n", ret);

out:
	printf("\n");
	printf("    Press any button to continue...\n");

	return ret;
}

bool __Check_HBC(void){
	u32 *stub = (u32 *)0x80001800;

	// Check HBC stub
	if (*stub)
		return true;
	return false;
}


int __Config_Game(void){
	int i, hotSpot, hotSpotPrev, button, ret;
	f32 size=0.0;
	bool repaint=true;

	//Obtener informacion del juego
	struct discHdr *header;
	header = &gameList[orden[selectedGame]];
	configuracionJuego *config=&configuracionJuegos[orden[selectedGame]];
	unsigned char* gameId=header->id;
	size=__Menu_GetGameSize(header);


	__Draw_Window(600, 380);
	MRC_Draw_Tile(120, 200, textures[TEX_CONTAINER], textures[TEX_CONTAINER]->width/2, 0);
	MRC_Draw_Texture(120, 200, config->banner);
	sprintf(tempString, "%s", gameId);
	MRC_Draw_String(100, 300, 0x505050ff, tempString);

	sprintf(tempString, "%s: %.2fGB", lang[36], size);
	MRC_Draw_String(76, 320, 0xa0a0a0ff, tempString);

	// Create/restore hotspots
	Wpad_CleanHotSpots();
	for(i=0; i<6;i++){
		MRC_Draw_String(240, 86+i*48, BLACK, lang[24+i]);
		Wpad_AddHotSpot(i,
			430,
			80+i*48,
			150,
			32,
			(i==0? 8 : i-1),
			(i==5? 8 : i+1),
			i, i
		);
	}
	Wpad_AddHotSpot(6,
		50, 380,
		24, 32,
		5, 0,
		8, 7
	);
	Wpad_AddHotSpot(7,
		180, 380,
		180, 32,
		5, 0,
		6, 8
	);
	Wpad_AddHotSpot(8,
		430, 380,
		150, 32,
		5, 0,
		7, 6
	);
	Wpad_SetHotSpot(0);


	// Loop
	hotSpot=hotSpotPrev=-1;

	ret=MENU_SELECT_GAME;

	for(;;){
		hotSpot=Wpad_Scan();

		// If hot spot changed
		if(hotSpot!=hotSpotPrev || repaint){
			hotSpotPrev=hotSpot;

			if(config->modoVideo==0){
				sprintf(tempString, lang[33]);
			}else{
				sprintf(tempString, "%s", lang[29+config->modoVideo]);
			}
			__Draw_Button(0, tempString, hotSpot==0);
			__Draw_Button(1, (config->anti002? lang[33]:lang[32]), hotSpot==1);
			if(config->ocarina==0){
				sprintf(tempString, lang[33]);
			}else if(config->ocarina==1){
				sprintf(tempString, "%s.gct", gameId);
			}else{
				sprintf(tempString, "%s_%d.gct", gameId, config->ocarina);
			}
			__Draw_Button(2, tempString, hotSpot==2);
			if(config->altdol==0){
				sprintf(tempString, lang[33]);
			}else{
				//sprintf(tempString, "%s %d", lang[34], config->altdol);
				sprintf(tempString, "%s", select_dol_from_internal_db((char*)gameId, config->altdol)->name);
			}
			__Draw_Button(3, tempString, hotSpot==3);
			if(config->ios==0){
				sprintf(tempString, lang[33]);
			}else{
				sprintf(tempString, "%d", config->ios);
			}
			__Draw_Button(4, tempString, hotSpot==4);
			__Draw_Button(5, (haHabidoCambios? lang[11]:lang[33]), hotSpot==5);
			__Draw_Button(6, "X", hotSpot==6);
			__Draw_Button(7, lang[41], hotSpot==7);
			__Draw_Button(8, lang[0], hotSpot==8);
			repaint=false;
		}
		__Draw_Cursor(Wpad_GetWiimoteX(), Wpad_GetWiimoteY(), 0);

		button=Wpad_GetButton();
		if((button & WPAD_BUTTON_A) && hotSpot>-1){
			if(hotSpot==0){
				config->modoVideo++;
				if(config->modoVideo==3)
					config->modoVideo=0;
				repaint=true;
			}else if(hotSpot==1){
				config->anti002=!config->anti002;
				repaint=true;
			}else if(hotSpot==2){
				config->ocarina++;
				if(config->ocarina==4)
					config->ocarina=0;
				repaint=true;
			}else if(hotSpot==3){
				config->altdol++;
				if(select_dol_from_internal_db((char*)gameId, config->altdol)==0)
					config->altdol=0;
				repaint=true;
			}else if(hotSpot==4){
				if(config->ios==0)
					config->ios=245;
				else if(config->ios==250)
					config->ios=0;
				else
					config->ios++;
				repaint=true;
			}else if(hotSpot==5){
				haHabidoCambios=!haHabidoCambios;
				repaint=true;
			}else if(hotSpot==6){
				ret=MENU_UNINSTALL;
				break;
			}else if(hotSpot==7){
				ret=MENU_SHORTCUT;
				break;
			}else if(hotSpot==8){
				break;
			}
		}else if(button & WPAD_BUTTON_HOME || button & WPAD_BUTTON_B || button & WPAD_BUTTON_1){
			break;
		}
	}

	return ret;
}


#define HOME_BUTTON_X			208
#define HOME_BUTTON_Y			120
#define HOME_BUTTON_WIDTH		224
#define HOME_BUTTON_HEIGHT		56
#define HOME_BUTTON_SEPARATION	4
int __Home(void){
	int i, hotSpot, hotSpotPrev, button, ret;
	bool repaint=true;
	f32 used, free;

	WBFS_DiskSpace(&used, &free);

	// Create/restore hotspots
	Wpad_CleanHotSpots();
	for(i=0; i<4; i++){
		Wpad_AddHotSpot(i,
			HOME_BUTTON_X,
			HOME_BUTTON_Y+i*(HOME_BUTTON_HEIGHT+HOME_BUTTON_SEPARATION),
			HOME_BUTTON_WIDTH,
			HOME_BUTTON_HEIGHT,
			(i==0? 3 : i-1),
			(i==3? 0 : i+1),
			i, i
		);
	}
	Wpad_SetHotSpot(0);


	__Draw_Window(HOME_BUTTON_WIDTH+44, 320);

	sprintf(tempString, "IOS%d v%d", IOS_GetVersion(), IOS_GetRevision());
	MRC_Draw_String(10, 420, 0x505050ff, tempString);
	sprintf(tempString, "%s: %.2fGB %s: %.2fGB", lang[37], used, lang[38], free);
	MRC_Draw_String(10, 440, 0x505050ff, tempString);
	MRC_Draw_String(500, 420, 0x505050ff, "Mighty Loader v17.0");
	MRC_Draw_String(560, 440, 0x505050ff, "by Marc R.");

	if(gameCnt==0)
		MRC_Draw_String(200, 440, 0xffff00FF, lang[17]);

	// Loop
	hotSpot=hotSpotPrev=-1;

	ret=MENU_SELECT_GAME;
	for(;;){
		hotSpot=Wpad_Scan();

		// If hot spot changed
		if((hotSpot!=hotSpotPrev && hotSpot<4) || repaint){
			hotSpotPrev=hotSpot;

			for(i=0; i<4; i++){
				__Draw_Button(i, lang[i], hotSpot==i);
			}
			repaint=false;
		}
		__Draw_Cursor(Wpad_GetWiimoteX(), Wpad_GetWiimoteY(), 0);

		button=Wpad_GetButton();
		if((button & WPAD_BUTTON_A) && hotSpot>-1 && hotSpot<4){
			if(hotSpot==1)
				ret=MENU_INSTALL;
			else if(hotSpot==2)
				ret=MENU_DOWNLOAD;
			else if(hotSpot==3)
				ret=MENU_EXIT;
			break;
		}else if(button & WPAD_BUTTON_HOME || button & WPAD_BUTTON_B){
			break;
		}
	}

	return ret;
}

#define DOWNLOAD_WINDOW_X		52
#define DOWNLOAD_WINDOW_Y		180
#define DOWNLOAD_WINDOW_WIDTH	536
#define DOWNLOAD_WINDOW_HEIGHT	112
int __Download(void){
	int i, ret;
	char region[4];

	if(gameCnt==0)
		return MENU_HOME;

	__Draw_Window(DOWNLOAD_WINDOW_WIDTH, DOWNLOAD_WINDOW_HEIGHT);
	Fat_MakeDir(MIGHTY_PATH IMAGES_PREFIX);

	MRC_Draw_String(DOWNLOAD_WINDOW_X+50, DOWNLOAD_WINDOW_Y+50, BLACK, lang[5]);
	__Render_Screen();

	for(;;){
		ret=net_init();
		if(ret<0 && ret!=-EAGAIN){
			sprintf(tempString, "ERROR: I can't connect (ret=%d)", ret);
			__Draw_Window(DOWNLOAD_WINDOW_WIDTH, DOWNLOAD_WINDOW_HEIGHT);
			MRC_Draw_String(DOWNLOAD_WINDOW_X+50, DOWNLOAD_WINDOW_Y+50, BLACK, tempString);
			__Render_Screen();
			//net_deinit();
			Wpad_WaitButtons();
			return MENU_HOME;
		}
		if(ret==0) //consigo conexion
			break;
		sleep(5);
	}

	__Draw_Window(DOWNLOAD_WINDOW_WIDTH, DOWNLOAD_WINDOW_HEIGHT);
	MRC_Draw_String(DOWNLOAD_WINDOW_X+20, DOWNLOAD_WINDOW_Y+20, BLACK, lang[6]);
	__Render_Screen();

	for(i=0;i<gameCnt;i++){
		__Draw_Box(DOWNLOAD_WINDOW_X+20+4, DOWNLOAD_WINDOW_Y+52+4, (500/gameCnt)*i, 32, BLUE-0x80);
		__Render_Screen();
		struct discHdr *header = &gameList[i];
		sprintf(tempString, MIGHTY_PATH IMAGES_PREFIX "/%s.png", header->id);
		if(!Fat_CheckFile(tempString)){
			if(header->id[3]=='E'){
				region[0]='U';
				region[1]='S';
				region[2]=0;
			}else if(header->id[3]=='J'){
				region[0]='J';
				region[1]='A';
				region[2]=0;
			}else{
				region[0]='E';
				region[1]='N';
				region[2]=0;
			}
			sprintf(tempString, "http://wiitdb.com/wiitdb/artwork/cover/%s/%s.png", region, header->id);
			sprintf(tempString, "http://art.gametdb.com/wii/cover/%s/%s.png", region, header->id);

			u8* outbuf=NULL;
			u32 outlen=0;
			u32 http_status=0;

			ret=http_request(tempString, 1 << 31);
			if(!ret){
				continue;
			}

			ret = http_get_result(&http_status, &outbuf, &outlen); 

			if(outlen>32){//suficientes bytes
				sprintf(tempString, MIGHTY_PATH IMAGES_PREFIX "/%s.png", header->id);
				Fat_SaveFile(tempString, (void *)&outbuf,outlen);
			}

			if(outbuf!=NULL)
				free(outbuf);
		}
	}
	net_deinit();

	//free textures
	return MENU_HOME;
}


void __Error_Message(const char* message){
	__Draw_Window(DOWNLOAD_WINDOW_WIDTH, DOWNLOAD_WINDOW_HEIGHT);
	MRC_Draw_String(DOWNLOAD_WINDOW_X+50, DOWNLOAD_WINDOW_Y+50, BLACK, message);
	__Render_Screen();
	Wpad_WaitButtons();
}

int __Create_Shortcut(void){
	char *dolBuffer=NULL;
	char *pngBuffer=NULL;
	char *xmlBuffer=NULL;
	int ios;

	//Obtener informacion del juego
	struct discHdr *header;
	header = &gameList[orden[selectedGame]];
	configuracionJuego *config=&configuracionJuegos[orden[selectedGame]];

	if(config->ios == 0){
		ios=IOS_GetVersion();
	}else{
		ios=config->ios;
	}
	int ret;

	if(__Question_Window(lang[42], lang[12], lang[11])){
		sprintf(tempString, "fat:/apps/%s", header->id);

		Fat_MakeDir(tempString);
		ret = Fat_ReadFile("fat:/apps/mighty_loader/boot.dol", (void *)&dolBuffer);
		if(ret != 1){
			sprintf(tempString, "fat:/apps/%s/boot.dol", header->id);
			Fat_SaveFile(tempString, (void *)&dolBuffer, ret);
			free(dolBuffer);

			xmlBuffer=malloc(1024);
			sprintf(tempString, "fat:/apps/%s/meta.xml", header->id);
			sprintf(xmlBuffer, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n<app version=\"1\">\n<name>  %s</name>\n<release_date>198605240000</release_date>\n<short_description>%s</short_description>\n<arguments>\n\t<arg>--autoload=%s</arg>\n\t<arg>--ios=%d</arg>\n\t<arg>--forceVideo=%d</arg>\n\t<arg>--cheatsFile=%d</arg>\n</arguments>\n<no_ios_reload/></app>\n", header->title, header->id, header->id, ios, config->modoVideo, config->ocarina);
			Fat_SaveFile(tempString, (void *)&xmlBuffer, strlen(xmlBuffer));
			free(xmlBuffer);

			sprintf(tempString, "fat:/apps/%s/icon.png", header->id);


			sprintf(tempString, MIGHTY_PATH "hbc_icons/%s.png", header->id);
			ret = Fat_ReadFile(tempString, (void *)&pngBuffer);
			if(ret != -1){
				sprintf(tempString, "fat:/apps/%s/icon.png", header->id);
				Fat_SaveFile(tempString, (void *)&pngBuffer, ret);
				free(pngBuffer);
				__Error_Message("OK!");
			}else{
				__Error_Message("OK! (WARNING: No icon found.)");
			}
			
		}else{
			__Error_Message("Mighty Loader boot.dol was not found.");
			__Error_Message("It must be installed in sd:/apps/mighty_loader");
		}

	}

	return MENU_CONFIG_GAME;
}


int Menu_Loop(unsigned char* id, int videoMode, int ocarina){
	int ret;
	bool autoMode=(id != 0);

	// Check console language
	if(CONF_GetLanguage()==CONF_LANG_SPANISH)
		lang=texts[1];
	else if(CONF_GetLanguage()==CONF_LANG_ITALIAN)
		lang=texts[2];
	else
		lang=texts[0];

	// Check if widescreen
	if(CONF_GetAspectRatio()==CONF_ASPECT_16_9)
		wideScreen=true;

	// Create needed directories
	if(!autoMode){
		Fat_MakeDir("fat:/config");
		Fat_MakeDir("fat:/config/mighty");
		Fat_MakeDir("fat:/config/mighty/hbc_icons");
		Fat_MakeDir("fat:/config/mighty/cheats");
	}

	// Load skin images
	__Load_Skin(!autoMode);

	// Init MRC graphics
	MRC_Init();

	// Open HDD
	ret=Open_Device();
	if(ret<0){
		ret=MENU_EXIT;
		goto out;
	}

	// Get game list
	if(!autoMode){
		__Menu_GetEntries();
		ret=MENU_SELECT_GAME;
	}else{
		orden=malloc(sizeof(u16));
		orden[0]=0;
		selectedGame=0;
		gameList=malloc(sizeof(struct discHdr));
		gameList[0].id[0]=id[0];
		gameList[0].id[1]=id[1];
		gameList[0].id[2]=id[2];
		gameList[0].id[3]=id[3];
		gameList[0].id[4]=id[4];
		gameList[0].id[5]=id[5];
		configuracionJuegos=malloc(sizeof(configuracionJuego));

		configuracionJuegos[0].modoVideo=videoMode;
		configuracionJuegos[0].anti002=true;
		configuracionJuegos[0].ocarina=ocarina;
		configuracionJuegos[0].altdol=0;
		configuracionJuegos[0].ios=0;

		gameCnt=1;
		ret=MENU_START_GAME;
	}

	for(;;){
		if(ret==MENU_SELECT_GAME)
			ret=__Select_Game();
		else if(ret==MENU_CONFIG_GAME)
			ret=__Config_Game();
		else if(ret==MENU_UNINSTALL)
			ret=__Uninstall_Game();
		else if(ret==MENU_SHOW_GAME)
			ret=__Show_Game(autoMode);
		else if(ret==MENU_HOME)
			ret=__Home();
		else if(ret==MENU_INSTALL)
			ret=__Install();
		else if(ret==MENU_DOWNLOAD)
			ret=__Download();
		else if(ret==MENU_SHORTCUT)
			ret=__Create_Shortcut();
		else
			break;
	}



	if(haHabidoCambios)
		__Save_Changes();

	if(ret==MENU_START_GAME){
		MRC_Draw_String(512, 440, WHITE, lang[43]);
		__Render_Box(512, 440);
		__Finish_ALL_GFX();
		__Start_Game();
	}


out:
	#ifdef DEBUG_MODE
	printf("Exiting...\n");		
	#endif

	if(!__Check_HBC())
		SYS_ResetSystem(SYS_RETURNTOMENU, 0, 0);

	return ret;
}
